@@ -2,11 +2,13 @@
 
 from .chunker import Chunk, Chunker
 from .chunker_langchain import LangChainChunker
+from .chunker_page import PageChunker
 from .chunker_unstructured import ChunkerUnstructured
 
 __all__ = [
     "Chunker",
     "Chunk",
     "LangChainChunker",
+    "PageChunker",
     "ChunkerUnstructured",
 ]
