@@ -1,4 +1,4 @@
-"""Configuration management for the PDF Knowledgebase server."""
+"""Configuration management for the Document Knowledgebase server (PDF and Markdown support)."""
 
 import logging
 import os
@@ -15,21 +15,30 @@ logger = logging.getLogger(__name__)
 
 @dataclass
 class ServerConfig:
-    """Runtime configuration for the PDF Knowledgebase server."""
+    """Runtime configuration for the Document Knowledgebase server."""
 
     # Required settings
     openai_api_key: str
 
     # Optional settings with defaults
-    knowledgebase_path: Path = field(default_factory=lambda: Path("./pdfs"))
+    knowledgebase_path: Path = field(default_factory=lambda: Path("./documents"))
     cache_dir: Path = field(default_factory=lambda: Path(""))
     chunk_size: int = 1000
     chunk_overlap: int = 200
+    min_chunk_size: int = 0  # Global minimum chunk size (0 = disabled)
     embedding_model: str = "text-embedding-3-large"
     embedding_batch_size: int = 100
 
+    # Transport configuration
+    transport: str = "stdio"  # "stdio" or "http"
+    server_host: str = "localhost"
+    server_port: int = 8000  # Used for both MCP HTTP and web interface
+
     # Embedding provider configuration
-    embedding_provider: str = "local"  # "local" or "openai"
+    embedding_provider: str = "local"  # "local", "openai", or "huggingface"
+
+    # OpenAI configuration
+    openai_api_base: Optional[str] = None  # Custom base URL for OpenAI-compatible endpoints
 
     # Local embedding configuration
     local_embedding_model: str = "Qwen/Qwen3-Embedding-0.6B"  # High quality 0.6B model
@@ -41,13 +50,26 @@ class ServerConfig:
     model_cache_dir: str = "~/.cache/pdfkb-mcp/models"
     embedding_dimension: int = 0  # 0 uses model default
     fallback_to_openai: bool = False
+
+    # GGUF configuration for quantized models
+    gguf_quantization: str = "Q6_K"  # Default quantization (Q8_0, F16, Q6_K, Q4_K_M, etc.)
+
+    # HuggingFace embedding configuration
+    huggingface_embedding_model: str = "sentence-transformers/all-MiniLM-L6-v2"  # Default lightweight model
+    huggingface_provider: Optional[str] = None  # Provider like "nebius", None for auto/default
+    huggingface_api_key: Optional[str] = None  # HF token, can also use HF_TOKEN env var
     vector_search_k: int = 5
     file_scan_interval: int = 60
     log_level: str = "INFO"
-    supported_extensions: List[str] = field(default_factory=lambda: [".pdf"])
+    supported_extensions: List[str] = field(default_factory=lambda: [".pdf", ".md", ".markdown"])
     unstructured_pdf_processing_strategy: str = "fast"
-    pdf_parser: str = "pymupdf4llm"
-    pdf_chunker: str = "langchain"
+
+    # Markdown-specific configuration
+    markdown_parse_frontmatter: bool = True  # Parse YAML/TOML frontmatter
+    markdown_extract_title: bool = True  # Extract title from first H1
+    pdf_parser: str = "pymupdf4llm"  # For PDFs only, markdown files bypass parsing
+    document_chunker: str = "langchain"  # Used for both PDFs and Markdown
+    pdf_chunker: str = "langchain"  # Backward compatibility alias
     docling_config: Dict[str, Any] = field(default_factory=dict)
     # Marker LLM configuration
     marker_use_llm: bool = False
@@ -69,6 +91,8 @@ class ServerConfig:
         default_factory=lambda: {"vector": 0.6, "text": 0.4}  # Semantic search weight  # BM25 search weight
     )
     rrf_k: int = 60  # RRF constant parameter
+    hybrid_expansion_factor: float = 2.0  # How much to expand limit for better fusion
+    hybrid_max_expanded_limit: int = 30  # Maximum expanded limit for hybrid search
 
     # Whoosh-specific configuration
     whoosh_index_dir: str = ""  # Auto-set to cache_dir/whoosh
@@ -84,22 +108,64 @@ class ServerConfig:
     semantic_chunker_min_chunk_size: Optional[int] = None
     semantic_chunker_min_chunk_chars: Optional[int] = 100
 
+    # Page chunking configuration
+    page_chunker_min_chunk_size: Optional[int] = 100
+    page_chunker_max_chunk_size: Optional[int] = None
+    page_chunker_merge_small: bool = True
+
+    # Markdown page boundary configuration
+    markdown_page_boundary_pattern: str = r"--\[PAGE:\s*(\d+)\]--"  # Default page pattern for markdown
+    markdown_split_on_page_boundaries: bool = True
+
     # Parallel processing configuration
     max_parallel_parsing: int = 1  # Max number of PDFs to parse simultaneously (default: 1 to prevent overload)
     max_parallel_embedding: int = 1  # Max number of documents to embed simultaneously (default: 1 to prevent overload)
     background_queue_workers: int = 2  # Total background queue workers
     thread_pool_size: int = 1  # Thread pool size for CPU-intensive operations
 
+    # Reranker configuration
+    enable_reranker: bool = False  # Disabled by default for backwards compatibility
+    reranker_provider: str = "local"  # Provider: local or deepinfra
+    reranker_model: str = "Qwen/Qwen3-Reranker-0.6B"  # Default reranker model
+    reranker_sample_additional: int = 5  # Additional results to sample for reranking
+    reranker_device: str = ""  # Auto-detect or specific device for reranker (local only)
+    reranker_model_cache_dir: str = "~/.cache/pdfkb-mcp/reranker"  # Cache directory for reranker models
+    reranker_gguf_quantization: str = ""  # GGUF quantization for reranker (e.g., Q6_K, Q8_0)
+
+    # DeepInfra configuration
+    deepinfra_api_key: str = ""  # API key for DeepInfra reranker
+    deepinfra_reranker_model: str = "Qwen/Qwen3-Reranker-8B"  # DeepInfra model: 0.6B, 4B, or 8B
+
+    # Document summarization configuration
+    enable_summarizer: bool = False  # Enable/disable document summarization
+    summarizer_provider: str = "local"  # Provider: local or remote
+    summarizer_model: str = "gpt-4"  # Model for summarization (remote) or local model name
+    summarizer_max_pages: int = 10  # Maximum pages to use for summarization
+    summarizer_device: str = ""  # Auto-detect or specific device for local summarizer
+    summarizer_model_cache_dir: str = "~/.cache/pdfkb-mcp/summarizer"  # Cache directory
+    summarizer_api_base: str = ""  # Custom API base URL for remote summarizer
+    summarizer_api_key: str = ""  # API key for remote summarizer (fallback to openai_api_key)
+
     def __post_init__(self):
         """Validate configuration after initialization."""
         # Set cache_dir relative to knowledgebase_path if not explicitly provided
         if not self.cache_dir or self.cache_dir == Path(""):
             self.cache_dir = self.knowledgebase_path / ".cache"
 
+        # Backward compatibility: sync pdf_chunker with document_chunker
+        if hasattr(self, "pdf_chunker") and not hasattr(self, "document_chunker"):
+            self.document_chunker = self.pdf_chunker
+        elif hasattr(self, "document_chunker"):
+            self.pdf_chunker = self.document_chunker
+
         # Set whoosh_index_dir if not explicitly provided
         if not self.whoosh_index_dir:
             self.whoosh_index_dir = str(self.cache_dir / "whoosh")
 
+        # Set default summarizer model based on provider if not explicitly set
+        if self.summarizer_model == "gpt-4" and self.summarizer_provider == "local":
+            self.summarizer_model = "Qwen/Qwen3-4B-Instruct-2507-FP8"
+
         self._validate_config()
         self._ensure_directories()
 
@@ -109,12 +175,22 @@ class ServerConfig:
         if self.embedding_provider == "openai":
             if not self.openai_api_key:
                 raise ConfigurationError("OPENAI_API_KEY is required when using OpenAI embeddings")
-            if not self.openai_api_key.startswith("sk-"):
-                raise ConfigurationError("Invalid OpenAI API key format")
+            # Only validate standard OpenAI key format if using the default endpoint
+            if not self.openai_api_base and not self.openai_api_key.startswith("sk-"):
+                raise ConfigurationError("Invalid OpenAI API key format (must start with 'sk-')")
         elif self.embedding_provider == "local":
             # For local embeddings, we can use a dummy key
             if not self.openai_api_key:
                 self.openai_api_key = "sk-local-embeddings-dummy-key"
+        elif self.embedding_provider == "huggingface":
+            # For HuggingFace embeddings, check for HF token
+            if not self.huggingface_api_key and not os.getenv("HF_TOKEN"):
+                raise ConfigurationError(
+                    "HF_TOKEN environment variable or huggingface_api_key is required when using HuggingFace embeddings"
+                )
+            # For OpenAI API key, use a dummy one if not provided
+            if not self.openai_api_key:
+                self.openai_api_key = "sk-huggingface-embeddings-dummy-key"
         else:
             raise ConfigurationError(f"Invalid embedding_provider: {self.embedding_provider}")
 
@@ -124,9 +200,23 @@ class ServerConfig:
         if self.chunk_overlap < 0:
             raise ConfigurationError("chunk_overlap cannot be negative")
 
+        # Validate transport configuration
+        if self.transport not in ["stdio", "http", "sse"]:
+            raise ConfigurationError("transport must be 'stdio', 'http', or 'sse'")
+
+        # Validate HTTP/SSE configuration
+        if self.transport in ["http", "sse"]:
+            if self.server_port <= 0 or self.server_port > 65535:
+                raise ConfigurationError("server_port must be between 1 and 65535")
+            if not self.server_host:
+                raise ConfigurationError("server_host cannot be empty")
+
         if self.chunk_overlap >= self.chunk_size:
             raise ConfigurationError("chunk_overlap must be less than chunk_size")
 
+        if self.min_chunk_size < 0:
+            raise ConfigurationError("min_chunk_size cannot be negative")
+
         if self.embedding_batch_size <= 0:
             raise ConfigurationError("embedding_batch_size must be positive")
 
@@ -162,8 +252,23 @@ class ServerConfig:
                 "pdf_parser must be either 'unstructured', 'pymupdf4llm', 'mineru', 'marker', 'docling', or 'llm'"
             )
 
-        if self.pdf_chunker not in ["langchain", "unstructured", "semantic"]:
-            raise ConfigurationError("pdf_chunker must be 'langchain', 'unstructured', or 'semantic'")
+        if self.pdf_chunker not in ["langchain", "unstructured", "semantic", "page"]:
+            raise ConfigurationError("pdf_chunker must be 'langchain', 'unstructured', 'semantic', or 'page'")
+
+        # Validate page chunking configuration if using page chunker
+        if self.pdf_chunker == "page":
+            if self.page_chunker_min_chunk_size and self.page_chunker_min_chunk_size <= 0:
+                raise ConfigurationError("page_chunker_min_chunk_size must be positive")
+
+            if self.page_chunker_max_chunk_size and self.page_chunker_max_chunk_size <= 0:
+                raise ConfigurationError("page_chunker_max_chunk_size must be positive")
+
+            if (
+                self.page_chunker_min_chunk_size
+                and self.page_chunker_max_chunk_size
+                and self.page_chunker_min_chunk_size >= self.page_chunker_max_chunk_size
+            ):
+                raise ConfigurationError("page_chunker_min_chunk_size must be less than max_chunk_size")
 
         # Validate semantic chunking configuration if using semantic chunker
         if self.pdf_chunker == "semantic":
@@ -283,6 +388,9 @@ class ServerConfig:
         if os.getenv("OPENAI_API_KEY") and not os.getenv("PDFKB_OPENAI_API_KEY"):
             logger.warning("OPENAI_API_KEY is deprecated, use PDFKB_OPENAI_API_KEY instead")
 
+        # Get OpenAI API base URL (for OpenAI-compatible endpoints)
+        openai_api_base = os.getenv("PDFKB_OPENAI_API_BASE")
+
         # Only require OpenAI key if using OpenAI embeddings
         if embedding_provider == "openai" and not openai_api_key:
             raise ConfigurationError(
@@ -299,6 +407,9 @@ class ServerConfig:
             "embedding_provider": embedding_provider,
         }
 
+        if openai_api_base:
+            config_kwargs["openai_api_base"] = openai_api_base
+
         # Parse optional path settings
         knowledgebase_path = os.getenv("PDFKB_KNOWLEDGEBASE_PATH") or os.getenv("KNOWLEDGEBASE_PATH")
         if os.getenv("KNOWLEDGEBASE_PATH") and not os.getenv("PDFKB_KNOWLEDGEBASE_PATH"):
@@ -331,6 +442,13 @@ class ServerConfig:
             except ValueError:
                 raise ConfigurationError(f"Invalid PDFKB_CHUNK_OVERLAP: {chunk_overlap}")
 
+        min_chunk_size = os.getenv("PDFKB_MIN_CHUNK_SIZE")
+        if min_chunk_size:
+            try:
+                config_kwargs["min_chunk_size"] = int(min_chunk_size)
+            except ValueError:
+                raise ConfigurationError(f"Invalid PDFKB_MIN_CHUNK_SIZE: {min_chunk_size}")
+
         embedding_batch_size = os.getenv("PDFKB_EMBEDDING_BATCH_SIZE") or os.getenv("EMBEDDING_BATCH_SIZE")
         if os.getenv("EMBEDDING_BATCH_SIZE") and not os.getenv("PDFKB_EMBEDDING_BATCH_SIZE"):
             logger.warning("EMBEDDING_BATCH_SIZE is deprecated, use PDFKB_EMBEDDING_BATCH_SIZE instead")
@@ -390,6 +508,23 @@ class ServerConfig:
             except ValueError:
                 raise ConfigurationError(f"Invalid PDFKB_EMBEDDING_CACHE_SIZE: {embedding_cache_size}")
 
+        gguf_quantization = os.getenv("PDFKB_GGUF_QUANTIZATION")
+        if gguf_quantization:
+            config_kwargs["gguf_quantization"] = gguf_quantization
+
+        # Parse HuggingFace embedding configuration
+        huggingface_embedding_model = os.getenv("PDFKB_HUGGINGFACE_EMBEDDING_MODEL")
+        if huggingface_embedding_model:
+            config_kwargs["huggingface_embedding_model"] = huggingface_embedding_model
+
+        huggingface_provider = os.getenv("PDFKB_HUGGINGFACE_PROVIDER")
+        if huggingface_provider:
+            config_kwargs["huggingface_provider"] = huggingface_provider
+
+        huggingface_api_key = os.getenv("PDFKB_HUGGINGFACE_API_KEY") or os.getenv("HF_TOKEN")
+        if huggingface_api_key:
+            config_kwargs["huggingface_api_key"] = huggingface_api_key
+
         max_sequence_length = os.getenv("PDFKB_MAX_SEQUENCE_LENGTH")
         if max_sequence_length:
             try:
@@ -573,17 +708,56 @@ class ServerConfig:
             except ValueError:
                 raise ConfigurationError(f"Invalid PDFKB_MINERU_VRAM: {mineru_vram}")
 
-        # Parse PDF chunker selection
-        pdf_chunker = os.getenv("PDFKB_PDF_CHUNKER") or os.getenv("PDF_CHUNKER")
-        if os.getenv("PDF_CHUNKER") and not os.getenv("PDFKB_PDF_CHUNKER"):
-            logger.warning("PDF_CHUNKER is deprecated, use PDFKB_PDF_CHUNKER instead")
-        if pdf_chunker:
-            chunker = pdf_chunker.lower()
-            if chunker not in ["langchain", "unstructured", "semantic"]:
+        # Parse document chunker selection (supports both new and old env vars)
+        document_chunker = (
+            os.getenv("PDFKB_DOCUMENT_CHUNKER") or os.getenv("PDFKB_PDF_CHUNKER") or os.getenv("PDF_CHUNKER")
+        )
+        if os.getenv("PDF_CHUNKER") and not os.getenv("PDFKB_PDF_CHUNKER") and not os.getenv("PDFKB_DOCUMENT_CHUNKER"):
+            logger.warning("PDF_CHUNKER is deprecated, use PDFKB_DOCUMENT_CHUNKER or PDFKB_PDF_CHUNKER instead")
+        if document_chunker:
+            chunker = document_chunker.lower()
+            if chunker not in ["langchain", "unstructured", "semantic", "page"]:
                 raise ConfigurationError(
-                    f"Invalid PDFKB_PDF_CHUNKER: {pdf_chunker}. Must be 'langchain', 'unstructured', or 'semantic'"
+                    f"Invalid PDFKB_DOCUMENT_CHUNKER/PDFKB_PDF_CHUNKER: {document_chunker}. "
+                    "Must be 'langchain', 'unstructured', 'semantic', or 'page'"
                 )
-            config_kwargs["pdf_chunker"] = chunker
+            config_kwargs["document_chunker"] = chunker
+            config_kwargs["pdf_chunker"] = chunker  # Backward compatibility
+
+        # Parse page chunking configuration
+        if page_min_size := os.getenv("PDFKB_PAGE_CHUNKER_MIN_CHUNK_SIZE"):
+            try:
+                config_kwargs["page_chunker_min_chunk_size"] = int(page_min_size) if page_min_size else None
+            except ValueError:
+                raise ConfigurationError(f"Invalid PDFKB_PAGE_CHUNKER_MIN_CHUNK_SIZE: {page_min_size}")
+
+        if page_max_size := os.getenv("PDFKB_PAGE_CHUNKER_MAX_CHUNK_SIZE"):
+            try:
+                config_kwargs["page_chunker_max_chunk_size"] = int(page_max_size) if page_max_size else None
+            except ValueError:
+                raise ConfigurationError(f"Invalid PDFKB_PAGE_CHUNKER_MAX_CHUNK_SIZE: {page_max_size}")
+
+        if page_merge := os.getenv("PDFKB_PAGE_CHUNKER_MERGE_SMALL"):
+            config_kwargs["page_chunker_merge_small"] = page_merge.lower() == "true"
+
+        # Parse markdown configuration
+        if os.getenv("PDFKB_MARKDOWN_PARSE_FRONTMATTER"):
+            config_kwargs["markdown_parse_frontmatter"] = (
+                os.getenv("PDFKB_MARKDOWN_PARSE_FRONTMATTER", "true").lower() == "true"
+            )
+
+        if os.getenv("PDFKB_MARKDOWN_EXTRACT_TITLE"):
+            config_kwargs["markdown_extract_title"] = (
+                os.getenv("PDFKB_MARKDOWN_EXTRACT_TITLE", "true").lower() == "true"
+            )
+
+        if markdown_page_pattern := os.getenv("PDFKB_MARKDOWN_PAGE_BOUNDARY_PATTERN"):
+            config_kwargs["markdown_page_boundary_pattern"] = markdown_page_pattern
+
+        if os.getenv("PDFKB_MARKDOWN_SPLIT_ON_PAGE_BOUNDARIES"):
+            config_kwargs["markdown_split_on_page_boundaries"] = (
+                os.getenv("PDFKB_MARKDOWN_SPLIT_ON_PAGE_BOUNDARIES", "true").lower() == "true"
+            )
 
         # Parse semantic chunking configuration
         if semantic_threshold_type := os.getenv("PDFKB_SEMANTIC_CHUNKER_THRESHOLD_TYPE"):
@@ -671,6 +845,18 @@ class ServerConfig:
             except ValueError:
                 raise ConfigurationError(f"Invalid PDFKB_RRF_K: {rrf_k}")
 
+        if hybrid_expansion_factor := os.getenv("PDFKB_HYBRID_EXPANSION_FACTOR"):
+            try:
+                config_kwargs["hybrid_expansion_factor"] = float(hybrid_expansion_factor)
+            except ValueError:
+                raise ConfigurationError(f"Invalid PDFKB_HYBRID_EXPANSION_FACTOR: {hybrid_expansion_factor}")
+
+        if hybrid_max_expanded_limit := os.getenv("PDFKB_HYBRID_MAX_EXPANDED_LIMIT"):
+            try:
+                config_kwargs["hybrid_max_expanded_limit"] = int(hybrid_max_expanded_limit)
+            except ValueError:
+                raise ConfigurationError(f"Invalid PDFKB_HYBRID_MAX_EXPANDED_LIMIT: {hybrid_max_expanded_limit}")
+
         # Parse parallel processing configuration
         if max_parallel_parsing := os.getenv("PDFKB_MAX_PARALLEL_PARSING"):
             try:
@@ -696,6 +882,82 @@ class ServerConfig:
             except ValueError:
                 raise ConfigurationError(f"Invalid PDFKB_THREAD_POOL_SIZE: {thread_pool_size}")
 
+        # Parse reranker configuration
+        if enable_reranker := os.getenv("PDFKB_ENABLE_RERANKER"):
+            config_kwargs["enable_reranker"] = enable_reranker.lower() in ["true", "1", "yes"]
+
+        if reranker_provider := os.getenv("PDFKB_RERANKER_PROVIDER"):
+            config_kwargs["reranker_provider"] = reranker_provider.lower()
+
+        if reranker_model := os.getenv("PDFKB_RERANKER_MODEL"):
+            config_kwargs["reranker_model"] = reranker_model
+
+        if reranker_sample_additional := os.getenv("PDFKB_RERANKER_SAMPLE_ADDITIONAL"):
+            try:
+                config_kwargs["reranker_sample_additional"] = int(reranker_sample_additional)
+            except ValueError:
+                raise ConfigurationError(f"Invalid PDFKB_RERANKER_SAMPLE_ADDITIONAL: {reranker_sample_additional}")
+
+        if reranker_device := os.getenv("PDFKB_RERANKER_DEVICE"):
+            config_kwargs["reranker_device"] = reranker_device
+
+        if reranker_model_cache_dir := os.getenv("PDFKB_RERANKER_MODEL_CACHE_DIR"):
+            config_kwargs["reranker_model_cache_dir"] = reranker_model_cache_dir
+
+        if reranker_gguf_quantization := os.getenv("PDFKB_RERANKER_GGUF_QUANTIZATION"):
+            config_kwargs["reranker_gguf_quantization"] = reranker_gguf_quantization
+
+        # DeepInfra configuration
+        if deepinfra_api_key := os.getenv("PDFKB_DEEPINFRA_API_KEY"):
+            config_kwargs["deepinfra_api_key"] = deepinfra_api_key
+
+        if deepinfra_reranker_model := os.getenv("PDFKB_DEEPINFRA_RERANKER_MODEL"):
+            config_kwargs["deepinfra_reranker_model"] = deepinfra_reranker_model
+
+        # Summarization configuration
+        if enable_summarizer := os.getenv("PDFKB_ENABLE_SUMMARIZER"):
+            config_kwargs["enable_summarizer"] = enable_summarizer.lower() in ("true", "1", "yes", "on")
+
+        if summarizer_provider := os.getenv("PDFKB_SUMMARIZER_PROVIDER"):
+            config_kwargs["summarizer_provider"] = summarizer_provider.lower()
+
+        if summarizer_model := os.getenv("PDFKB_SUMMARIZER_MODEL"):
+            config_kwargs["summarizer_model"] = summarizer_model
+
+        if summarizer_max_pages := os.getenv("PDFKB_SUMMARIZER_MAX_PAGES"):
+            try:
+                config_kwargs["summarizer_max_pages"] = int(summarizer_max_pages)
+            except ValueError:
+                raise ConfigurationError(f"Invalid PDFKB_SUMMARIZER_MAX_PAGES: {summarizer_max_pages}")
+
+        if summarizer_device := os.getenv("PDFKB_SUMMARIZER_DEVICE"):
+            config_kwargs["summarizer_device"] = summarizer_device
+
+        if summarizer_model_cache_dir := os.getenv("PDFKB_SUMMARIZER_MODEL_CACHE_DIR"):
+            config_kwargs["summarizer_model_cache_dir"] = summarizer_model_cache_dir
+
+        if summarizer_api_base := os.getenv("PDFKB_SUMMARIZER_API_BASE"):
+            config_kwargs["summarizer_api_base"] = summarizer_api_base
+
+        if summarizer_api_key := os.getenv("PDFKB_SUMMARIZER_API_KEY"):
+            config_kwargs["summarizer_api_key"] = summarizer_api_key
+
+        # Parse transport configuration
+        transport = os.getenv("PDFKB_TRANSPORT")
+        if transport:
+            config_kwargs["transport"] = transport.lower()
+
+        server_host = os.getenv("PDFKB_SERVER_HOST") or os.getenv("PDFKB_SSE_HOST")
+        if server_host:
+            config_kwargs["server_host"] = server_host
+
+        server_port = os.getenv("PDFKB_SERVER_PORT") or os.getenv("PDFKB_SSE_PORT")
+        if server_port:
+            try:
+                config_kwargs["server_port"] = int(server_port)
+            except ValueError:
+                raise ConfigurationError(f"Invalid PDFKB_SERVER_PORT: {server_port}")
+
         return cls(**config_kwargs)
 
     @property
