@@ -4,12 +4,14 @@ import asyncio
 import json
 import logging
 import time
+import warnings
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, Dict, Optional
 
 from fastmcp import FastMCP
 
 from .config import ServerConfig
+from .document_processor import DocumentProcessor
 from .embeddings import EmbeddingService
 from .exceptions import (
     ConfigurationError,
@@ -22,7 +24,6 @@ from .exceptions import (
 from .file_monitor import FileMonitor
 from .intelligent_cache import IntelligentCacheManager
 from .models import Document, SearchQuery
-from .pdf_processor import PDFProcessor
 from .vector_store import VectorStore
 
 if TYPE_CHECKING:
@@ -45,9 +46,11 @@ class PDFKnowledgebaseServer:
         """
         self.config = config or ServerConfig.from_env()
         self.app = FastMCP("PDF Knowledgebase")
-        self.pdf_processor: Optional[PDFProcessor] = None
+        self.document_processor: Optional[DocumentProcessor] = None
         self.vector_store: Optional[VectorStore] = None
         self.embedding_service: Optional[EmbeddingService] = None
+        self.reranker_service = None
+        self.summarizer_service = None
         self.file_monitor: Optional[FileMonitor] = None
         self.cache_manager: Optional[IntelligentCacheManager] = None
         self.background_queue = background_queue
@@ -79,30 +82,71 @@ class PDFKnowledgebaseServer:
             self.embedding_service = EmbeddingService(self.config)
             await self.embedding_service.initialize()
 
+            # Initialize reranker service if enabled
+            if self.config.enable_reranker:
+                from .reranker_factory import get_reranker_service
+
+                try:
+                    self.reranker_service = get_reranker_service(self.config)
+                    if self.reranker_service:
+                        await self.reranker_service.initialize()
+                        logger.info("Reranker service initialized successfully")
+                except Exception as e:
+                    logger.error(f"Failed to initialize reranker service: {e}")
+                    logger.warning("Continuing without reranker")
+                    self.reranker_service = None
+
+            # Initialize summarizer service if enabled
+            self.summarizer_service = None
+            if self.config.enable_summarizer:
+                try:
+                    from .summarizer_factory import create_summarizer_service
+
+                    self.summarizer_service = create_summarizer_service(self.config)
+                    if self.summarizer_service:
+                        await self.summarizer_service.initialize()
+                        logger.info("Summarizer service initialized successfully")
+                except Exception as e:
+                    logger.error(f"Failed to initialize summarizer service: {e}")
+                    logger.warning("Continuing without summarizer")
+                    self.summarizer_service = None
+
             self.vector_store = VectorStore(self.config)
             self.vector_store.set_embedding_service(self.embedding_service)
+            self.vector_store.set_reranker_service(self.reranker_service)
             await self.vector_store.initialize()
 
-            self.pdf_processor = PDFProcessor(
-                self.config, self.embedding_service, self.cache_manager, self._embedding_semaphore
+            self.document_processor = DocumentProcessor(
+                self.config,
+                self.embedding_service,
+                self.cache_manager,
+                self._embedding_semaphore,
+                self.summarizer_service,
             )
 
             # Log startup configuration summary for diagnostics
             try:
                 parser_name = (
-                    type(self.pdf_processor.parser).__name__ if self.pdf_processor else str(self.config.pdf_parser)
+                    type(self.document_processor.parser).__name__
+                    if self.document_processor
+                    else str(self.config.pdf_parser)
                 )
                 chunker_name = (
-                    type(self.pdf_processor.chunker).__name__ if self.pdf_processor else str(self.config.pdf_chunker)
+                    type(self.document_processor.chunker).__name__
+                    if self.document_processor
+                    else str(self.config.document_chunker)
                 )
             except Exception:
                 parser_name = str(self.config.pdf_parser)
                 chunker_name = str(self.config.pdf_chunker)
+            reranker_info = f"Enabled ({self.config.reranker_model})" if self.config.enable_reranker else "Disabled"
             logger.info(
-                "Startup configuration: Parser=%s, Chunker=%s, EmbeddingModel=%s, KnowledgebasePath=%s, CacheDir=%s",
+                "Startup configuration: Parser=%s, Chunker=%s, EmbeddingModel=%s, Reranker=%s, "
+                "KnowledgebasePath=%s, CacheDir=%s",
                 parser_name,
                 chunker_name,
                 self.config.embedding_model,
+                reranker_info,
                 self.config.knowledgebase_path,
                 self.config.cache_dir,
             )
@@ -135,7 +179,7 @@ class PDFKnowledgebaseServer:
 
             self.file_monitor = FileMonitor(
                 self.config,
-                self.pdf_processor,
+                self.document_processor,
                 self.vector_store,
                 self._update_document_cache,
                 background_queue=self.background_queue,
@@ -275,7 +319,7 @@ class PDFKnowledgebaseServer:
             logger.info("Re-processing existing cached documents with new configuration...")
 
             # Re-process cached markdown files (skips expensive parsing)
-            documents = await self.pdf_processor.reprocess_cached_documents()
+            documents = await self.document_processor.reprocess_cached_documents()
 
             if documents:
                 # Add all documents to vector store
@@ -345,15 +389,15 @@ class PDFKnowledgebaseServer:
 
         @self.app.tool()
         async def add_document(path: str, metadata: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
-            """Add a PDF document to the knowledgebase for searching and analysis.
+            """Add a document (PDF or Markdown) to the knowledgebase for searching and analysis.
 
-            Use this tool to ingest new PDF documents. Once added, the document content
-            will be automatically processed, chunked, and made searchable via search_documents.
-            You do not need to call any other tools after adding - the document becomes
-            immediately available for searching.
+            Use this tool to ingest new documents. Supports both PDF and Markdown (.md, .markdown) files.
+            Once added, the document content will be automatically processed, chunked, and made searchable
+            via search_documents. You do not need to call any other tools after adding - the document
+            becomes immediately available for searching.
 
             Args:
-                path: Path to the PDF file to add to the knowledgebase.
+                path: Path to the document file (PDF or Markdown) to add to the knowledgebase.
                 metadata: Optional metadata to associate with the document (e.g., tags, categories).
 
             Returns:
@@ -368,15 +412,18 @@ class PDFKnowledgebaseServer:
                 if not file_path.exists():
                     raise ValidationError(f"File does not exist: {path}", "path")
 
-                if not file_path.suffix.lower() == ".pdf":
-                    raise ValidationError(f"File is not a PDF: {path}", "path")
+                supported_extensions = [".pdf", ".md", ".markdown"]
+                if not file_path.suffix.lower() in supported_extensions:
+                    raise ValidationError(
+                        f"File type not supported. Must be one of {supported_extensions}: {path}", "path"
+                    )
 
                 logger.info(f"Adding document: {path}")
                 start_time = time.time()
 
                 # Process the PDF with semaphore to limit parallelism
                 async with self._parsing_semaphore:
-                    result = await self.pdf_processor.process_pdf(file_path, metadata)
+                    result = await self.document_processor.process_document(file_path, metadata)
 
                 if not result.success:
                     logger.error(f"Failed to process PDF {path}: {result.error}")
@@ -725,7 +772,7 @@ class PDFKnowledgebaseServer:
 
         @self.app.resource("pdf://{document_id}")
         async def get_document(document_id: str) -> str:
-            """Get a PDF document by ID.
+            """Get a document (PDF or Markdown) by ID.
 
             Args:
                 document_id: ID of the document to retrieve.
@@ -752,7 +799,7 @@ class PDFKnowledgebaseServer:
 
         @self.app.resource("pdf://{document_id}/page/{page_number}")
         async def get_document_page(document_id: str, page_number: int) -> str:
-            """Get a specific page of a PDF document.
+            """Get a specific page of a document (PDF only, not applicable to Markdown).
 
             Args:
                 document_id: ID of the document.
@@ -788,7 +835,7 @@ class PDFKnowledgebaseServer:
 
         @self.app.resource("pdf://list")
         async def list_all_documents() -> str:
-            """List all available PDF documents.
+            """List all available documents (PDFs and Markdown files).
 
             Returns:
                 JSON string with document list and metadata.
@@ -850,7 +897,7 @@ class PDFKnowledgebaseServer:
                     logger.info(f"Auto-processing {change_type} PDF: {file_path}")
 
                     # Process the PDF
-                    result = await self.pdf_processor.process_pdf(file_path)
+                    result = await self.document_processor.process_document(file_path)
 
                     if result.success and result.document:
                         # Update vector store
@@ -962,8 +1009,33 @@ class PDFKnowledgebaseServer:
         """Run the MCP server."""
         await self.initialize()
 
-        # Use run_async() instead of run() to work within existing event loop
-        await self.app.run_async()
+        if self.config.transport in ["http", "sse"]:
+            transport_name = "HTTP" if self.config.transport == "http" else "SSE"
+            logger.info(
+                f"Running MCP server in {transport_name} mode on {self.config.server_host}:{self.config.server_port}"
+            )
+
+            await self.app.run_http_async(
+                transport=self.config.transport,
+                host=self.config.server_host,
+                port=self.config.server_port,
+                show_banner=True,
+            )
+        else:
+            logger.info("Running MCP server in stdio mode")
+            # Use run_async() instead of run() to work within existing event loop
+            await self.app.run_async()
+
+    def sync_run(self) -> None:
+        """Synchronous wrapper for run() - runs in a separate thread with its own event loop."""
+        logger.info("Starting MCP sync_run in thread")
+        try:
+            asyncio.run(self.run())
+        except Exception as e:
+            logger.error(f"MCP sync_run error: {e}")
+            raise
+        finally:
+            logger.info("MCP sync_run completed in thread")
 
     async def shutdown(self) -> None:
         """Shutdown the server gracefully."""
@@ -988,6 +1060,7 @@ class PDFKnowledgebaseServer:
 def main():
     """Entry point for the MCP server."""
     import argparse
+    import signal
     import sys
 
     # Parse command line arguments
@@ -1007,15 +1080,35 @@ Environment Variables:
   LOG_LEVEL              Logging level (default: INFO)
 
 Examples:
-  pdfkb-mcp                          # Run with default settings (MCP only, web disabled)
+  pdfkb-mcp                          # Run with default settings (MCP only, stdio transport)
+  pdfkb-mcp --transport http         # Run with HTTP transport (for Cline, modern clients)
+  pdfkb-mcp --transport sse          # Run with SSE transport (for Roo, legacy clients)
   PDFKB_WEB_ENABLE=true pdfkb-mcp    # Run with web interface enabled
   pdfkb-mcp --config myconfig.env    # Use custom config file
+
+Remote Transport:
+  HTTP (modern): http://localhost:8000/mcp/
+  SSE (legacy):  http://localhost:8000/sse/
+
+Integrated Mode:
+  With web enabled, both MCP and web run on adjacent ports:
+  Web: http://localhost:8084/ (web-port, from PDFKB_WEB_PORT)
+  MCP: http://localhost:8085/[mcp|sse]/ (web-port + 1, depends on transport)
+  API Docs: http://localhost:8084/docs
         """,
     )
 
     parser.add_argument("--config", type=str, help="Path to environment configuration file")
 
     parser.add_argument("--port", type=int, help="Override MCP server port (for stdio mode, this has no effect)")
+    parser.add_argument(
+        "--transport",
+        choices=["stdio", "http", "sse"],
+        default="stdio",
+        help="MCP transport mode (default: stdio, use http/sse for remote connections)",
+    )
+    parser.add_argument("--server-host", type=str, help="Host for HTTP transport mode (default: localhost)")
+    parser.add_argument("--server-port", type=int, help="Port for HTTP transport mode (default: 8000)")
 
     parser.add_argument("--enable-web", action="store_true", help="Enable web interface")
     parser.add_argument("--log-level", choices=["DEBUG", "INFO", "WARNING", "ERROR"], help="Override logging level")
@@ -1034,9 +1127,25 @@ Examples:
     # Load main configuration
     config = ServerConfig.from_env()
 
+    # Log full configuration for debugging
+    logger.info("Loaded configuration details:")
+    for key, value in config.__dict__.items():
+        if isinstance(value, Path):
+            logger.info(f"  {key}: {value}")
+        elif isinstance(value, (list, dict)):
+            logger.info(f"  {key}: {value}")
+        else:
+            logger.info(f"  {key}: {value}")
+
     # Override configuration from command line arguments
     if args.enable_web:
         config.web_enabled = True
+    if args.transport:
+        config.transport = args.transport
+    if getattr(args, "server_host", None):
+        config.server_host = args.server_host
+    if getattr(args, "server_port", None):
+        config.server_port = args.server_port
     if args.log_level:
         config.log_level = args.log_level
 
@@ -1044,12 +1153,34 @@ Examples:
     log_level = getattr(logging, config.log_level.upper(), logging.INFO)
     logging.basicConfig(level=log_level, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s")
 
+    # Suppress websockets deprecation warnings from third-party libraries
+    # These warnings are from uvicorn and fastmcp using deprecated websockets APIs
+    # and are not actionable by end users
+    warnings.filterwarnings(
+        "ignore", message="websockets.legacy is deprecated.*", category=DeprecationWarning, module="websockets.legacy"
+    )
+    warnings.filterwarnings(
+        "ignore",
+        message=".*WebSocketServerProtocol is deprecated.*",
+        category=DeprecationWarning,
+        module="uvicorn.protocols.websockets.websockets_impl",
+    )
+
     logger.info("Starting PDF Knowledgebase MCP Server")
     logger.info(f"Version: {__import__('pdfkb').__version__}")
     logger.info(f"Configuration: {config.knowledgebase_path}")
     logger.info(f"Cache directory: {config.cache_dir}")
     logger.info(f"Web interface: {'enabled' if config.web_enabled else 'disabled'}")
 
+    # Set up signal handlers for graceful shutdown
+    def signal_handler(signum, frame):
+        logger.info(f"Received signal {signum}, initiating graceful shutdown...")
+        # This will cause KeyboardInterrupt to be raised in the event loop
+        raise KeyboardInterrupt(f"Signal {signum}")
+
+    signal.signal(signal.SIGINT, signal_handler)
+    signal.signal(signal.SIGTERM, signal_handler)
+
     try:
         if config.web_enabled:
             # Run integrated server (MCP + Web)
@@ -1069,10 +1200,12 @@ Examples:
             server = PDFKnowledgebaseServer(config)
             asyncio.run(server.run())
     except KeyboardInterrupt:
-        logger.info("Received interrupt signal, shutting down...")
+        logger.info("Received interrupt signal, shutting down gracefully...")
     except Exception as e:
         logger.error(f"Server error: {e}")
         sys.exit(1)
+    finally:
+        logger.info("Shutdown complete")
 
 
 if __name__ == "__main__":
